@@ -33,6 +33,38 @@ pub const SECS_PER_DAY: f64 = SECS_PER_HOUR * 24.0;
 /// Number of seconds in a year.
 pub const SECS_PER_YEAR: f64 = SECS_PER_DAY * 365.0;
 
+/// A fallible conversion from one duration representation to another.
+///
+/// This is very similar to the `std::convert::TryFrom` trait which is currently
+/// unstable.
+pub trait FromDuration<T>: Sized {
+    type Error;
+    /// Convert `from` into a `Self` object.
+    fn from_duration(from: T) -> Result<Self, Self::Error>;
+}
+
+/// A fallible conversion that consumes `self`.
+///
+/// This is very similar to the `std::convert::TryInto` trait which is currently
+/// unstable.
+///
+/// Similar to `std::convert::Into`, this trait is reflexively implemented for
+/// all implementations of `FromDuration` and should not be manually implemented.
+pub trait IntoDuration<T>: Sized {
+    type Error;
+    /// Convert `self` into a `T` object.
+    fn into_duration(self) -> Result<T, Self::Error>;
+}
+
+impl<T, U> IntoDuration<U> for T
+    where U: FromDuration<T>
+{
+    type Error = U::Error;
+    fn into_duration(self) -> Result<U, U::Error> {
+        U::from_duration(self)
+    }
+}
+
 /// A specific point in time.
 ///
 /// Types implementing `TimePoint` can have a `FloatDuration` computed between them
@@ -355,6 +387,49 @@ impl TimePoint for time::SystemTime {
     }
 }
 
+impl FromDuration<time::Duration> for FloatDuration {
+    type Error = ();
+    #[inline]
+    fn from_duration(from: time::Duration) -> Result<FloatDuration, ()> {
+        Ok(FloatDuration::from_std(from))
+    }
+}
+#[cfg(feature = "chrono")]
+impl FromDuration<chrono::Duration> for FloatDuration {
+    type Error = ();
+    #[inline]
+    fn from_duration(from: chrono::Duration) -> Result<FloatDuration, ()> {
+        Ok(FloatDuration::from_chrono(from))
+    }
+}
+impl FromDuration<FloatDuration> for time::Duration {
+    type Error = error::OutOfRangeError;
+    #[inline]
+    fn from_duration(from: FloatDuration) -> Result<time::Duration, error::OutOfRangeError> {
+        from.to_std()
+    }
+}
+#[cfg(feature = "chrono")]
+impl FromDuration<FloatDuration> for chrono::Duration {
+    type Error = error::OutOfRangeError;
+    #[inline]
+    fn from_duration(from: FloatDuration) -> Result<chrono::Duration, error::OutOfRangeError> {
+        from.to_chrono()
+    }
+}
+
+impl From<time::Duration> for FloatDuration {
+    fn from(from: time::Duration) -> FloatDuration {
+        FloatDuration::from_std(from)
+    }
+}
+#[cfg(feature = "chrono")]
+impl From<chrono::Duration> for FloatDuration {
+    fn from(from: chrono::Duration) -> FloatDuration {
+        FloatDuration::from_chrono(from)
+    }
+}
+
 impl fmt::Display for FloatDuration {
     fn fmt(&self, fmt: &mut fmt::Formatter) -> fmt::Result {
         if self.secs > SECS_PER_YEAR {
@@ -371,8 +446,13 @@ impl fmt::Display for FloatDuration {
             write!(fmt, "{} milliseconds", self.as_milliseconds())
         } else if self.secs > 1.0e-6 {
             write!(fmt, "{} microseconds", self.as_microseconds())
-        } else {
+        } else if self.secs > 1.0e-9 {
             write!(fmt, "{} nanoseconds", self.as_nanoseconds())
+        } else if self.is_zero() {
+            write!(fmt, "0 seconds")
+        } else {
+            // Here we simply print seconds in scientific notation.
+            write!(fmt, "{:e} seconds", self.as_seconds())
         }
     }
 }
@@ -606,7 +686,7 @@ mod tests {
         assert_eq!(format!("{}", FloatDuration::days(3.0) + FloatDuration::hours(12.0)),
                    "3.5 days");
         assert_eq!(format!("{}", FloatDuration::seconds(12.7)), "12.7 seconds");
-        assert_eq!(format!("{}", FloatDuration::default()), "0 nanoseconds");
+        assert_eq!(format!("{}", FloatDuration::default()), "0 seconds");
 
         assert_eq!(format!("{}", FloatDuration::microseconds(100.0)),
                    "100 microseconds");
@@ -623,6 +703,8 @@ mod tests {
                    "25.25 nanoseconds");
         assert_eq!(format!("{}", FloatDuration::minutes(90.0)), "1.5 hours");
         assert_eq!(format!("{}", FloatDuration::years(2.5)), "2.5 years");
+        assert_eq!(format!("{}", FloatDuration::seconds(1.5e-30)),
+                   "1.5e-30 seconds");
     }
 
     #[test]
